/* INTERCAMBIO DE DATOS ARDUINO (actuando como servidor) y LABVIEW (actuando como cliente).

   Se leen i=8 señales analógicas (0-5V) conectadas a los pins A0-A7 de Arduino y se 
   transmiten mediante un transmisor/receptor Bluetooth conectado a los pins TX14 y RX15.
   
   Los datos se transmiten en serie a través del puerto Serial (correspondiente al 
   transmisor/receptor Bluetooth) de Arduino que funciona a 19200 bps sin salto de linea. 

   Dichos datos se estructuran conforme al protocolo establecido de la siguiente manera
   y son transmitidos y recibidos como caracteres ASCII:
       
        <L| N_SECUENCIA | CODIGO_DE_SEGURIDAD |L | COMANDO_DATOS | Li | DATOSi (2bytes/dato)| L| CONTROL DE ERROR | >
        
        Donde: < = Inicio de trama (1byte).
               L = Longitud (en bytes) del carácter proximo (1byte equivalente a un caracter ASCII).
               N_SECUENCIA = Número de la secuencia (1byte).
               CODIGO_DE_SEGURIDAD = Código de seguridad (1byte = 206)
               COMANDO_DATOS = Selección de modo de funcionamiento (1 byte):
                               COMANDO: El cliente manda comandos que ejecuta Arduino, tales como
                               seleccionar que señales se transmiten, modificar el tiempo de refresco,
                               encender una alarma (LED), mandar un echo de la trama recibida....
                               
                               DATOS: Se transmiten todas las señales de forma normal.
             
              DATOSi = Datos de la variable i (valores entre 0-1023 correspondientes a una variable de 2bytes).
              CONTROL DE ERROR: Suma de los valores de todo lo enviado anteriormente a excepción de los simbolos
              de inicio y fin de trama y el valor del propio error.
              
   El envio se realiza mediante un proceso de pregunta-respuesta, por lo que hasta obtener respuesta, arduino manda
   indefinidamente la secuencia inicial 0.
 */

/////////////////////////////////
//VARIABLES GLOBALES          //
///////////////////////////////

  #define PowerLED 23   //seleción del PIN en el que se encuentra conectado el LED ROJO (Power);
  #define LED_Amarillo 22 ///seleción del PIN en el que se encuentra conectado el LED AMARILLO (ALARMA);
  #define c_seg 206    //codigo de seguridad = 206;
  
  unsigned int n_seq = 0;        // variable para almacenar el nº de secuencia
  byte c_d = 0;        // Comandos o datos a ejecutar o continuar 
  
  byte Sensor_PIN[] = {A7, A6, A5, A4, A3, A2, A1, A0}; //ENTRADAS ANALÓGICAS
  unsigned int Sensor[8];  //Almacenamiento de los datos procedentes de las variables analógicas
  byte l_S[8];             //Longitud de los datos procedentes de las variables analógicas
  unsigned int retardo = 0;
  
  unsigned int dat_rec[8]; //Almacenamiento de los datos recibidos para analizalo y ejecutar los comandos que portan
  byte l_dat_rec[8];
  
  String datos_recibidos = "";    //String en el que se almacenan datos recibidos.
   boolean recepcion = false;
  
  
/////////////////////////////////
//DECLARACION DE FUNCIONES    //
///////////////////////////////

byte cuentaChars(unsigned long N);
byte l_error (int d_error);
unsigned int string2int (String objeto, int inicio, int fin);
String CAPTURA_DATOS (unsigned int c_d, unsigned int D1);
int CALCULA_ERROR(unsigned int n_seq, int c_d);
String CONSTRUIR_TRAMA(unsigned int n_seq, int c_d, String datos, int error);
void ENVIA_DATOS();
void ANALIZA_DATOS();
void RECIBE_DATOS();


/////////////////////////////////
//CONFIGURACIÓN               //
///////////////////////////////

void setup(){
  //Serial.begin(9600);
  Serial3.begin(19200);
  //Serial.setTimeout(1000);
  Serial3.setTimeout(60000);
  datos_recibidos.reserve(100); //Datos reservados para la recepcion de datos
  pinMode(PowerLED, OUTPUT);  //Pin PowerLED como salida
  pinMode(LED_Amarillo, OUTPUT); //Pin LED_Amarillo como salida
}


/////////////////////////////////
//PROGRAMA PRINCIPAL          //
///////////////////////////////

void loop(){
  
   if (Serial3.available()){  //RECIBE MIENTRAS EL PUERTO DE SERIE ESTE DISPONIBLE
   RECIBE_DATOS();
   }
   if (recepcion){  //SI HA RECIBIDO CORRECTAMENTE DATOS
     ANALIZA_DATOS();  //ANALIZA
     ENVIA_DATOS();   //ENVIA
     datos_recibidos="";  //RESET DE LOS DATOS RECIBIDOS
     recepcion=false;  //RESET DEL FLAG DE RECEPCION
   }
}
